﻿#!/usr/bin/perl
# letter_label.pl
# A letter label printing macro for Nisus Writer Express 2.x
# written by Nobumi Iyanaga
# (based on the Mail Merge.pl macro written by David T. Linker
# and modified by Kino)
# 
# The data file must have the following format -- each field delimited by a
# comma:

# dest,exp,FirstName,LastName,FirstLine,SecondLine,City,PostCode

# The first two words are used to distinguish the two columns of 
# the template file.  Other fields may be any thing.
# The subsequent lines must be:
# second line in the file for the first record in the left column
# third line in the file for the first record in the right column
# and so on...

# The template file must have the following format:

# «dest_FirstName» «dest_LastName»
# «dest_FirstLine»
# «dest_SecondLine»
# «dest_City»
#             «dest_PostCode»

# for the left column, and:

# «exp_FirstName» «exp_LastName»
# «exp_FirstLine»
# «exp_SecondLine»
# «exp_City»
#              «exp_PostCode»
# 

# for the right column.

#
# Save the this file ("letter_label.pl") in the macro folder in the folder
# <user home>/Library/Application Support/Nisus Writer/Macros
#
# Make a template file, with the keywords surrounded by
# guillemots, like this: «dest_keyword» or «exp_keyword». You can get these 
# characters by using <option>-\ and <option>-<shift>-\. Save the template
# file as [NOT plain text in UTF-8, BUT] rtf file, with the last part of the name
# before the extension being " template".
#
# Make a data file, with the first line consisting of the 
# keywords separated by commas, then the subsequent lines with 
# the text to substitute separated by commas. Save the file to the 
# same folder as the template file, with the same name as the template 
# file, but without the word "template". -- This file may be either an
# rtf file, or a plain text UTF-8 file.
#
# Leave this file open as the front window, and run the macro "letter_label".
# The output will be opened in a new window, and saved to the
# same directory as the other files.
#
#
# Valuable suggestions and additions by Kino, of the Nisus list

# Added code to make the output in rtf format for NWE 2.x. Nobumi Iyanaga 05-07-12

#Nisus Macro Block
#source front
#destination none
#End Nisus Macro Block

use strict;
use warnings;

use utf8;
binmode (STDIN, ":utf8");

# Is there a front window file, and has it been saved?
#if (@ARGV<1) {	# No arguments means no merge file
#	die "Front window not saved, or no front window\n";
#}

# Now, make the name of the input and output files
my $label_template_file = $ARGV[1];	# Get file path
# $label_template_file = '/Volumes/myDocument/Users/ni/Desktop/test_things/mail_merge_NWE/Label_test/Label2.rtf';

$label_template_file =~ s/\..{0,3}$//;	# Remove extension
my $label_output_file = $label_template_file.' OUTPUT.rtf';
$label_template_file = $label_template_file.' template.rtf';


# Read from template file
open (TEMPLATE,'<',$label_template_file) 
	or die "Unable to open $label_template_file";

my @template = <TEMPLATE>;
close(TEMPLATE);

# separate rtf header, with page header and footer, from rtf data

# make a string from the list of lines.
my $template_contents = join ("\n", grep {chomp} @template);

my ($rtfheader, $rtfdata, $rtffooter) = $template_contents =~ /^(\{\\rtf1.+?)(\{\\footer .+)(\}[\n\r]*)/s;

my $pagefooter = &extract_rtf_token ("footer", $rtfdata);

my $temppagefooter = quotemeta ($pagefooter);

$rtfdata =~ s/$temppagefooter//s;

$rtfheader = $rtfheader . $pagefooter;

# Read merge data from front window

# open (IN, "<:utf8", '/Volumes/myDocument/Users/ni/Desktop/test_things/mail_merge_NWE/Label_test/Label2.txt');
# my @labelfile = <IN>;
# close (IN);
my @labelfile = <STDIN>;

foreach my $line (@labelfile) {
	chomp ($line);
}

# Extract the keywords
my @keywords = split(/,/,$labelfile[0]);
my $left_col_key = shift (@keywords);
# the first character after « is encoded as rtf hex...
$left_col_key =~ s/^(.)/"(\\" . &convert_to_rtf_hex ($1). "|$1)"/e;
$left_col_key = "\\\\'c7" . $left_col_key . "_";

my $right_col_key = shift (@keywords);
# the first character after « is encoded as rtf hex...
$right_col_key =~ s/^(.)/"(\\" . &convert_to_rtf_hex ($1). "|$1)"/e;
$right_col_key = "\\\\'c7" . $right_col_key . "_";

foreach my $line (@keywords) {
	$line= $line . "\\\\'c8";
}

# Open the output file
open (OUTFILE,'>',$label_output_file)
	or die "Unable to create $label_output_file";
	
# print the rtf header to the output file
print OUTFILE $rtfheader, "\n";

#Do the actual merge
my $copy = $rtfdata;	# make a local copy to alter
for (my $i=1; $i<@labelfile;$i++) {
	my @subst = split(/,/,$labelfile[$i]);
	for (my $j = 0; $j < @subst;$j++) {
		my $prefix;
		if ($i % 2) {
			$prefix = $left_col_key;
		}
		else {
			$prefix = $right_col_key;
		}
		my $keyword = $prefix . $keywords[$j];
#  		print "copy: $copy\n";
#  		print "keyword: $keyword\tsubst: $subst[$j]\n";
		$subst[$j] =~ s/([^\x20-\x7F])/'\u' . unpack ("U", $1) . " ?"/ge;
		$copy =~ s/$keyword/$subst[$j]/s;
	}
}

print OUTFILE $copy;
print OUTFILE $rtffooter;

close (OUTFILE);

# Let's look at the result

# if you have installed /Developer...  Otherwise, comment out the line below.
`/Developer/Tools/SetFile -c NISX "$label_output_file"`;
`open -a "Nisus Writer Express" "$label_output_file"`;

sub convert_to_rtf_hex {
	my $char = shift;
	
	$char = sprintf ("\\'%02x", unpack ("U", $char));

	return ($char);
	
}

sub extract_rtf_token {
	my $keyword = shift;
	my $str = shift;
	
	if (! $str) {
		return ("");
	}

	my $res;

	my $num = $str =~ s/^(.*?)(\{\\$keyword.+)$/$2/s;
	if (! $num || ! $str) {
#		die ("sub extract_rtf_token couldn't find the keyword...");
		return ("");
	}

	my $pat = '.*?\}';
	($res) = $str =~/^($pat)/s;
	if (! $res) {
		die ("sub extract_rtf_token couldn't find the matching close brace...");
	}
	my $temp = $res;
	my $temp2 = $temp;
	if ($temp =~ s/\{//sg == $temp2 =~ s/\}//sg) {
		return $res;
	}
	else {
		$num = 0;
		while (1) {
			$num++;
			my $pat2 = $pat x $num;
			($res) = $str =~/^(\{$pat2)/s;
			if (! $res) {
				die ("sub extract_rtf_token couldn't find the matching close brace...");
			}
			$temp = $res;
			$temp2 = $temp;
			if ($temp =~ s/\{//sg == $temp2 =~ s/\}//sg) {
				return $res;
			}
		}
	}
}
